//<c>   Copyright 1987-2000 by Gerry J. Danen; all rights reserved.

//<t>   BU - Batch Utility

#include"gd_tools.h"
#include"gd_extrn.h"
#include"gd_scrn.h"
#include"gd_kb.h"


#define pgVERS_ "V4.20"
#define pgTITL_ "Batch Utility"
#define pgCOPR_ "1987"


// #define BU_DEBUG


#pragma check_stack(off)


#define RETURN          13      // <CR>
#define ESCAPE          27      // Escape
#define REG_MODE        TRUE    // TRUE for normal, FALSE for distribution BU
#define DEMO            FALSE   // TRUE for demo for BU_MENU
#define MAXCMD          133     // longest command line (+6)

/*
        Key codes:      P - Pause, wait for [Enter]             1)
                        C - Enter Character                     1)
                        K - Enter Character with timeout value  1)
                        Y - Enter Y or N                        1)
                        D - Dump File to Screen
                        F - Display File
                        H - Display Header                      1)
                        M - Display Message                     1) 2)
                        B - Display Blank Line                     2)
                        T - Display Day, Date, and Time            2)
                            & return week day number
                        S - Sound                                     3)
                        ^ - Clear the Screen, like DOS CLS
                        _ - Set Cursor ON, OFF, or BIG

        All codes may be followed by ^, which will clear the screen first, e.g. P^, Y^, etc.
        Code H and H^ work the same, and so do ^ and ^^.

        1) These codes may be followed by a message to be displayed.

        2) These functions use DOS for the display, so the information may be redirected to a file or printer.
           All other output does NOT use DOS.

        3) Follow this code by R(ed Alert), W(arble), N(oise), P(hone), G(lissando), or B(eep)
*/
#if REG_MODE
UCHAR   color_defs[] =
//      "Color codes in decimal: SCRFG=07 SCRBG=01 BOXFG=15 BOXBG=04 BOXBO=07 TIMFG=03 <---" ;
        "Color codes in decimal: SCRFG=07 SCRBG=01 BOXFG=04 BOXBG=07 BOXBO=01 TIMFG=03 <---" ;
//       01234567890123456789012345678901234567890123456789012345678901234567890123456
//                 1         2         3         4         5         6         7
#endif
UCHAR   *tbl_mth[] =
{
        "?",    // Illegal Month
        "January", "February", "March", "April", "May", "June", "July", "August",
        "September", "October", "November", "December"
};
UCHAR   *dump, *ln[25], scr_attr, str[9], cmdline[MAXCMD], strout[MAXCMD],
        deflt[MAXCMD], now_day[10], now_date[19], now_time[9], now_dates[12],
        now_times[12] ;
INT32   yyyymmdd, hhmmsstt, tics ;
INT16   yyyy, mm, dd, hh, ss, tt, last_tab ;
INT16   def_sound[] = { 400, 300, 400, 300, 800, 600, 800, 600, 0 };    // default sound
int     now_day_num ;
BOOL    tinkered_with_cursor = FALSE ;


//      Prototypes
void    bu_border(INT16 color);
void    bu_quit(int exitcode);
void    bu_sleep(INT16 millisecs);
void    do_bl_line(void);
void    do_char(void);
void    do_cursor(void);
void    do_dump(void);
void    do_file(void);
void    do_header(void);
void    do_help(void);
void    do_message(void);
void    do_pause(void);
void    do_sound(void);
void    do_timer(void);
void    do_wait(void);
void    do_yes_no(void);
void    dump_msg(INT16 r,INT16 c,STRING str);
void    dump_screen(void);
void    get_date_and_time(void);
void    get_vid_mode(void);
void    initialize_colors(void);
void    make_box(STRING text);
void    make_box_hor(INT16 row,UCHAR lch,UCHAR hch,UCHAR rch);
void    sound_error(void);
void    sound_glissando(int lower,int upper);
void    sound_prompt(void);
int     wait_time_or_any_key(INT16 secs);
int     what_day(INT16 yy,INT16 mm,INT16 dd,STRING dest);


#pragma optimize("", off)


void    bu_sleep( INT16 millisecs )
{
    static INT32 ll, speed=0 ;

    if ( ! speed )
    {
        speed = _speed_factor/8L ;
        if ( ! speed )
            speed = 10L ;
    }
    for ( ; millisecs ; millisecs-- )
    {
        for ( ll=speed ; ll ; ll-- )
            ;
    }
}

#pragma optimize("", on)


void    sound_prompt( void )
{
    Tone( 300, 12);
    Tone( 550, 8 );
    Tone( 800, 4 );
}


void    sound_glissando( int lower, int upper )
{
    ToneOn();
    do  {
        ToneSet( ToneFreq( lower ) );
        bu_sleep( 1 );
        lower += 3 ;
    }   while ( upper > lower );
    ToneOff();
}


void    sound_error( void )
{
    sound_glissando( 800, 1800 );
}


int     wait_time_or_any_key( INT16 secs )
{
    RC          c, r ;
    INT16       ch = 0, going = TRUE ;
    int         secs1 = -1, secs2 = 0 ;
    long        time_1, time_2 ;
    UCHAR       *bLine = NULPTR ;
    static UCHAR msg[41] ;

    c = sc_col();
    r = sc_row();
    bLine = SaveLine( 1, bLine );
    time_1 = time(0) + (long) secs ;

#ifdef BU_DEBUG
    printf( "\nIn Wait4Key routine, before loop\n" );
    getch();
#endif

    while ( going )
    {
#ifdef BU_DEBUG
    printf( "\nIn Wait4Key routine, ready for a key\n" );
    getch();
#endif
        if ( kb_Hit() )
        {
            InKey( &ch );
            going = FALSE ;
        }
        if ( going )
        {
            time_2 = time(0);
            if ( time_2 >= time_1 )
                going = FALSE ;
        }
        if ( going )
        {
            secs1 = (INT16) labs(time_1 - time_2);
            if ( secs1 != secs2 )
            {
                sprintf( msg, " Waiting %d second%s ", secs1, secs1==1 ? "" : "s" );
                DspMsg( 1,1, vid.err_bg, vid.err_fg, msg, NO );
                secs2 = secs1 ;
            }
//~~~       ch = 0 ;
        }
    }
    bLine = RestoreLine( 1, bLine );
    MoveCursor( r, c );
    return( ch );
}


void    bu_border( INT16 color )
{
    set_color( COLOR_BORDER, color, -1 );
    SetBorder();
}


void    get_vid_mode( void )
{
    R_REGS ;

    VideoModeRefresh();
    if ( vid.cur_page != 0 )
    {
        r.h.ah = 5 ;                            // set to page 0
        r.h.al = 0 ;
        video_call(&r);
        VideoModeRefresh();                     // get status again
    }
    last_tab = (vid.cols - 1) / 8 ;
    last_tab = (last_tab * 8) + 1 ;
}


void    bump_up( void )
{
    if ( sc_row() >= vid.rows )
    {
        sc_scroll( 1, vid.normal_attr, 1, 1, vid.rows, vid.cols, SCROLL_UP );
        MoveCursor( vid.rows, 1 );
    }
    else
        MoveCursor( sc_row()+1, 1 );
}


void    bu_quit( int exitcode )
{
    bu_border( vid.normal_bg );
    if ( ! tinkered_with_cursor )
        CursOn();
#ifdef BU_DEBUG
    printf( "\nExit code = [%d]\n", exitcode );
#endif
    exit( exitcode );
}


void    do_bl_line( void )
{
    printf( "\n" );
    ClearEOS();
}


void    make_box_hor( INT16 row, UCHAR lch, UCHAR hch, UCHAR rch )
{
    DspCh( row, 1, vid.sts_fg, vid.err_bg, lch, 1 );
    DspCh( row, 2, vid.sts_fg, vid.err_bg, hch, vid.cols-2 );
    DspCh( row, vid.cols, vid.sts_fg, vid.err_bg, rch, 1 );
}


void    make_box( STRING text )
{
    make_box_hor( 1, (UCHAR)'', (UCHAR)'', (UCHAR)'' );
    make_box_hor( 2, (UCHAR)'', (UCHAR)' ', (UCHAR)'' );
    make_box_hor( 3, (UCHAR)'', (UCHAR)'', (UCHAR)'' );
    DspMsg( 2, CenterBetween( 1,vid.cols,text ), vid.err_fg, vid.err_bg, text, NO );
}


int     what_day( INT16 yy, INT16 mm, INT16 dd, STRING dest )
{
    INT16       day_index ;
    INT32       y4, y100, y400, t1, t2, t3, t4 ;

    if ( mm < 3 )
    {
         mm = mm + 12 ;
         yy--;
    }
    y4   = yy / 4 ;
    y100 = yy / 100 ;
    y400 = yy / 400 ;
    t1   = 6 * (mm + 1);
    t1   = t1 / 10 ;
    t2   = dd + 2 * mm + t1 + yy + y4 - y100 + y400 + 2;
    t3   = t2 / 7 ;
    t3   = t3 * 1000;
    t4   = t2 * 1000 / 7 ;
    t4   = (t4 - t3) * 7 + 500 ;
    day_index = (INT16) (t4 / 1000) ;
    day_index = ( day_index == 0 ? 7 : day_index );
    st_cpy( dest, DayName(day_index), 10 );
    return( (int) day_index );
}


void    get_date_and_time( void )
{
    UCHAR       am_pm ;

    dt_getsystemdate( &yyyymmdd, &yyyy, &mm, &dd );
    now_day_num = what_day( yyyy, mm, dd, now_day );
    st_cpy ( str, tbl_mth[mm], 4 );
    sprintf( now_date , "%d %s %d", dd, tbl_mth[mm], yyyy );
    sprintf( now_dates, "%02d-%s-%d", dd, str, yyyy );
    tm_getsystemtime( &hhmmsstt, &tics, &hh, &mm, &ss, &tt );
    sprintf( now_times, "%02d:%02d:%02d.%02d", hh, mm, ss, tt );
    am_pm = (UCHAR) ((hh >= 12) ? 'p' : 'a') ;
    hh %= 12 ;
    if ( hh < 1 )
        hh += 12 ;
    sprintf( now_time, "%2d:%02d%cm", hh, mm, am_pm );
}


void    do_header( void )
{
    cls();
    st_cpy( strout, (strlen(cmdline) ? cmdline : pgm_author), vid.cols-3 );
    make_box( strout );
    DspMsg( 4, 2, vid.tt_time_fg, vid.normal_bg, now_day, NO );
    DspMsg( 4, CenterBetween( 1, vid.cols, now_date ), vid.tt_time_fg, vid.normal_bg, now_date, NO );
    DspMsg( 4, vid.cols-7, vid.tt_time_fg, vid.normal_bg, now_time, NO );
    MoveCursor( 6, 1 );
}


void    do_pause( void )
{
    INT16       ch, row, col ;
    UCHAR       *p1 = "Press the ", *p2 = " [Enter] ", *p3 = " key to continue, or ", *p4 = " [Ctrl]/[Break] ", *p5 = " to abort... " ;

    if ( strlen( cmdline ) )                    // got a message
    {
        do_bl_line();
        row = sc_row();
        DspMsg( row, 1, vid.normal_fg, vid.normal_bg, cmdline, NO );
    }
    do_bl_line();
    row = sc_row();
    DspMsg( row, 1,   vid.normal_fg, vid.normal_bg, p1, NO );  col = 1   + strlen( p1 );
    DspMsg( row, col, vid.normal_bg, vid.normal_fg, p2, NO );  col = col + strlen( p2 );
    DspMsg( row, col, vid.normal_fg, vid.normal_bg, p3, NO );  col = col + strlen( p3 );
    DspMsg( row, col, vid.normal_bg, vid.normal_fg, p4, NO );  col = col + strlen( p4 );
    DspMsg( row, col, vid.normal_fg, vid.normal_bg, p5, YES);
    FlushKeyboard();
    bu_border( WHITE );
    sound_prompt();
    do
    {
        InKey( &ch );
    }   while( ch != RETURN );
    do_bl_line();
}


void    do_message( void )
{
    do_bl_line();
    printf( "\r%s\n", cmdline );
    do_bl_line();
}


void    do_timer( void )
{
    st_cpy( str, now_day, 4 );
    printf( "%s %s %s\n", str, now_dates, now_times );
    bu_quit( now_day_num );
}


#pragma optimize("", off)

void    do_sound( void )
{
    INT16       i ;

    _strupr( cmdline );
    switch ( cmdline[0] )
    {
        case 'B' :                              // Beep
            sn_beep();
            break ;
        case 'G' :                              // Glissando
            sound_glissando( 800, 1200 );
            break ;
        case 'N' :                              // Noise
            for ( i=0 ; i < 5 ; i++ )
            {
                Tone(  300, 6 );
                Tone(  900, 6 );
            }
            break ;
        case 'P' :                              // Phone
            for ( i=0 ; i < 25 ; i++ )
            {
                Tone( 523, 3 );
                Tone( 659, 3 );
            }
            break ;
        case 'R' :                              // Red Alert
            for ( i=0 ; i < 3 ; i++ )
            {
                sound_glissando( 800, 1200 );
            }
            break ;
        case 'W' :                              // Warble
            for ( i=0 ; i < 12 ; i++ )
            {
                Tone( 3000, 3 );
                tm_sleep_msecs( 1 );
            }
            break ;
        default  :                              // Default sound
            i = 0 ;
            while ( def_sound[i] )
            {
                Tone( def_sound[i], 10 );
                i += 1 ;
            }
            break ;
    }
}


#pragma optimize("", on)


BOOL    extract_braced_parm( UCHAR in[], UCHAR parm[] )
{
    int x ;

    ST_CLR ( parm );
    st_trim( in );

    if ( in[0] != '{' )                         // no {} parameter
        return( NO );

    x = 1 ;
    while ( (in[x] != EOS) && (in[x] != '}') )
    {
        parm[x-1] = in[x] ;
        in[x] = ' ' ;
        x += 1 ;
    }
    parm[x-1] = EOS ;
    in[x] = in[0] = ' ' ;
    st_trim( in );
    _strupr( parm );

    return( YES );
}


RC      do_prompt( RC r, STRING p1, STRING p2, STRING defresp )
{
    RC  c ;

    DspMsg( r, 1, vid.sts_fg, vid.err_bg, p1, NO );   c = 1 + strlen( p1 );
    DspMsg( r, c, vid.sts_fg, vid.err_bg, cmdline,NO); c= c + strlen( cmdline );
    DspMsg( r, c, vid.sts_fg, vid.err_bg, p2, NO );   c = c + strlen( p2 ) - 2 ;
    DspMsg( r, c, vid.err_fg, vid.err_bg, defresp, NO );
    MoveCursor( r, c );
    bu_border( WHITE );
    sound_prompt();
    return( c );
}


void    do_char( void )
/*
        All lowercase letters are converted to uppercase letters, regardless of
        processing mode.

        If "cmdline" starts with "{" then assume that a list of acceptable
        characters follows, terminated with a "}".  In order for this procedure
        to work properly, all characters given must be unique.  If {abcA} is
        handed to the program, it will never return errorlevel 4!

        If such a list is given, then only a character from the list is allowed,
        and an errorlevel is returned corresponding to the position in the list.
        For example, if {ABC} is the given list, and the "B" was entered,
        errorlevel 2 is returned.

        If no list is given, then the ASCII value of the character is returned
        in the errorlevel.  If a "Q" is entered, the errorlevel is set to 81.

        If a second {?} is found, as in "BU C {abc} {B} Select" then the first
        char in the second list is the default response if [Enter] is pressed.
        Optionally, the second set may contain a timeout value in seconds,
        after which the default is returned.  Example: "BU C {abc} {B:15} Select"
        will wait for 15 seconds.  If no key is pressed within that time frame,
        "B" is returned.
*/
{
    INT16       ch, row, col, posn, delay=0, defChar=0 ;
    UCHAR       *p1 = " ", *p2 = " ...   ", resp[2] ;

    do_bl_line();
    row = sc_row();
    extract_braced_parm( cmdline, strout );
    extract_braced_parm( cmdline, deflt );
    if ( strlen(deflt) )
    {
        defChar = resp[0] = deflt[0] ;
        deflt[0] = ' ' ;
    }
    else
        resp[0] = '_' ;
    resp[1] = EOS ;
    st_trim( deflt );
    if ( strlen(deflt) > 1 && deflt[0] == ':' )
        delay = atoi( &deflt[1] );

    col = do_prompt( row, p1, p2, resp );

    if ( strlen( strout ) )
    {
        posn = -1 ;
        while ( posn < 0 )
        {
            if ( delay )
            {
                ch = wait_time_or_any_key( delay );
                if ( ! ch )
                    ch = RETURN ;
            }
            else
                InKey( &ch );
            ch = toupper( ch );
            if ( ch == RETURN && defChar )
                ch = defChar ;

            posn = StrFindChar( (UCHAR) ch, strout );
            if ( posn < 0 )
                sound_error();
        }
    }
    else
    {
        InKey( &ch );
    }
    resp[0] = (UCHAR) ch ;
    resp[1] = EOS ;
    _strupr( resp );
    DspMsg( row, col, vid.err_fg, vid.err_bg, resp, NO );
    bump_up();
    bump_up();
    if ( strlen( strout ) )
        bu_quit( posn + 1 );
    else
        bu_quit( (int) resp[0] );
}


void    do_yes_no( void )
{
    INT16       resp = -1, ch, row, col, delay=0, defChar=0 ;
    UCHAR       *p1 = " ", p2[] = " (Y/N)?   ", defResp = '_' ;

/*
    Formats:

    bu y Continue
    bu y {y} Continue
    bu y {y:15} Continue
*/

    do_bl_line();
    row = sc_row();

    extract_braced_parm( cmdline, deflt );
    if ( deflt[0] == 'Y' || deflt[0] == 'N' )
    {
        if ( deflt[0] == 'Y' )
            p2[4] = 'n' ;
        else
            p2[2] = 'y' ;
        defChar = deflt[0] ;
        if ( deflt[1] == ':' && strlen(&deflt[2]) )
            delay = atoi( &deflt[2] );
        deflt[1] = EOS ;
    }
    else
    {
        strcpy( deflt, "_" );
    }

    col = do_prompt( row, p1, p2, deflt );

    do  {
        if ( delay )
        {
            ch = wait_time_or_any_key( delay );
            if ( ! ch )
                ch = RETURN ;
        }
        else
            InKey( &ch );
        ch = toupper( ch );
        if ( ch == RETURN && defChar )
            ch = defChar ;
        if ( ch == (INT16) 'Y' )
            resp = 1 ;
        if ( ch == (INT16) 'N' )
            resp = 0 ;
        if ( resp < 0 )
            sound_error();
    }   while( resp < 0 );
    DspMsg( row, col, vid.err_fg, vid.err_bg, (resp ? "Yes " : "No "), NO );
    bump_up();
    bump_up();
    bu_quit( resp );
}


void    do_cursor( void )
{
    if ( st_eq( cmdline, "OFF" ) )
        CursOff();
    else
    if ( st_eq( cmdline, "BIG" ) || st_eq( cmdline, "BOX" ) )
        CursOnBox();
    else
        CursOn();
    tinkered_with_cursor = TRUE ;
}


void    initialize_colors( void )
{
#if REG_MODE
    if ( (vid.mode == 1) || (vid.mode == 3) )   // color burst mode
    {
        vid.normal_fg  = atoi( &color_defs[ 30 ] );
        vid.normal_bg  = atoi( &color_defs[ 39 ] );
        vid.err_fg     = atoi( &color_defs[ 48 ] );
        vid.err_bg     = atoi( &color_defs[ 57 ] );
        vid.sts_fg     = atoi( &color_defs[ 66 ] ); // for box border
        vid.sts_bg     = vid.err_bg ;
        vid.tt_time_fg = atoi( &color_defs[ 75 ] );
    }
    else                                        // black & white modes
    {
#endif
        vid.normal_fg  = vid.err_bg = vid.sts_bg = WHITE ;
        vid.normal_bg  = vid.err_fg = vid.sts_fg = BLACK ;
        vid.tt_time_fg = BR_WHITE ;
#if REG_MODE
    }
#endif
    set_color( COLOR_NORMAL , vid.normal_fg , vid.normal_bg );
    set_color( COLOR_ERROR  , vid.err_fg    , vid.err_bg );
    set_color( COLOR_STATUS , vid.sts_fg    , vid.sts_bg );
    set_color( COLOR_TT_TIME, vid.tt_time_fg, -1 );
}


#if REG_MODE
GD_DET hlp[] = {
        21,  8, 0, "Refer to manual for more information",
        18, 20, 0, "_: Set Cursor ON, OFF, or BIG",
        17, 20, 0, "^: Clear the Screen",
        16, 20, 0, "W: Wait Seconds",
        15, 20, 0, "S: Sounds <B/G/N/P/R/W/X>",
        14, 20, 0, "T: Display Day, Date, and Time & Return Week Day Number",
        13, 20, 0, "B: Display Blank Line",
        12, 20, 0, "M: Display Message",
        11, 20, 0, "H: Display Header",
        10, 20, 0, "F: Display File",
         9, 20, 0, "D: Dump File to Screen",
         8, 20, 0, "Y: Enter Y or N           {def:time}",
         7, 20, 0, "C: Enter Character {list} {def:time}",
         6,  8, 0, "Key codes-> P: Pause, wait for [Enter]",
         0,  0, 0, "" };
#endif


void    do_help( void )
{
    INT16       i=0 ;
    STRING      pH ;

    pH = strchr( pgm_copyrite, ';' );           // find "all rights reserved"
    if ( pH )
    {
        *pH = EOS ;
        st_trim( pgm_copyrite );
    }

    sprintf( cmdline, "%s %s   %s", pgm_title, pgm_vers, pgm_copyrite );
    do_header();

    sound_error();

#if REG_MODE
#if DEMO
#else
    while ( strlen( hlp[i].msg ) )
    {
        DspMsg( hlp[i].row, hlp[i].col, vid.normal_fg, vid.normal_bg, hlp[i].msg, NO );
        i += 1 ;
    }
    MoveCursor( 23,1 );
#endif
#endif
    bu_quit( 1 );
}


#if REG_MODE
void    dump_screen( void )
{
    INT16       l ;

    LOOP(  l , 0, 24, 1 )
    {
        if ( ln[ l ] != NULL )
        {
            tty_msg( ln[ l ], vid.normal_fg, vid.normal_bg );
            free( ln[ l ] );
            ln[ l ] = (UCHAR *) NULL ;
        }
    }
}


void    do_file( void )
{
    FILE        *fdin ;
    INT16       ch = 0, lcnt = 0 ;

    if ( ( fdin = fopen( cmdline, "r" ) ) == NULL )
    {
        printf( "\nFile \"%s\" not found\n", cmdline );
        sound_error();
        bu_quit( 1 );
    }
    CursOff();
    FOREVER
    {
        if ( fgets( cmdline, (MAXCMD-1), fdin ) == NULL )
            goto done ;
        if ( lcnt < 25 )
        {
            if ( (ln[lcnt] = malloc( strlen(cmdline)+2 )) == NULL )
            {
                tty_msg( cmdline, vid.normal_fg, vid.normal_bg );
                lcnt = 999 ;
                sn_beep();
            }
            else
            {
                st_cpy( ln[lcnt], cmdline, MAXCMD );
                lcnt += 1 ;
            }
        }
        if ( lcnt == 25 )
        {
            dump_screen();
            lcnt = 0 ;
            if ( kb_Hit() )
                InKey( &ch );
            if ( ch == ESCAPE )
                goto done ;
        }
    }
done:
    dump_screen();
    CursOn();
}


void    dump_msg( INT16 r, INT16 c, STRING str )
{
    UCHAR       *ptr ;

    r -= 1 ;                                    // use 0-offset for row &
    c -= 1 ;                                    //                  column
    ptr = dump + (((r * vid.cols) + c) * 2);
    while ( *str )
    {
        *ptr++ = *str++ ;
        *ptr++ ;
    }
}


void    do_dump( void )
{
    FILE        *fdin ;
    INT16       row ;

    row = sc_row();
    if ( (dump = malloc( vid.rows * vid.cols * 2 )) == NULL )
    {
        strcpy( cmdline, "Not enough memory for screen dump" );
        do_pause();
        bu_quit( 1 );
    }
    if ( ( fdin = fopen( cmdline, "r" ) ) == NULL )
    {
        sprintf( dump, "File \"%s\" not found!", cmdline );
        strcpy( cmdline, dump );
        do_pause();
        bu_quit( 1 );
    }
    ScBlockRead( dump, 1,1, vid.rows, vid.cols );
    CursOff();
    FOREVER
    {
        if ( row > vid.rows )
            goto do_dump_display ;
        if ( fgets( cmdline, 132, fdin ) == NULL )
            goto do_dump_display ;
        if ( cmdline[ strlen(cmdline)-1 ] == '\n' )
            cmdline[ strlen(cmdline)-1 ] = EOS ;
        StrDetab( strout, cmdline, vid.cols );
        dump_msg( row, 1, strout );
        row += 1 ;
    }
do_dump_display:
    ScBlockWrite( dump, 1,1, vid.rows, vid.cols );
    CursOn();
    if ( row > vid.rows )
        row = vid.rows ;
    MoveCursor( row, 1 );
}
#endif


void    do_wait( void )
{
    INT16       secs ;

    secs = atoi( cmdline );
    wait_time_or_any_key( secs );               // allow user abort
    CursOn();
}


void    main( void )
{
    UCHAR       key_code, clr_code ;

    fs_init( TRUE, SHAREWARE_VERSION, 401, pgVERS_, pgTITL_, pgCOPR_ ); // leave cursor
    kb_Break( BREAK_ON );
    CmdLine( cmdline );                         // get the command line
    st_trim( cmdline );
    if ( ! strlen( cmdline ) )
        strcpy( cmdline, "?" );                 // assume ? if "" entered

    if ( cmdline[0] != 'W' && cmdline[0] != 'w' )
        CursOff();                              // leave cursor for WAIT

    get_vid_mode();                             // must do again
    st_cpy( strout, cmdline, 3 );               // extract first two chars
    _strupr( strout );
    key_code = strout[0] ;
    clr_code = strout[1] ;
    cmdline[0] = cmdline[1] = ' ';
    st_trim( cmdline );
    initialize_colors();
    if ( (clr_code == '^') || (key_code == '^') )
        cls();
    get_date_and_time();
    if ( key_code != 'W' )                      // WAIT does it's own thing
        CursOn();
    switch ( key_code )
    {
#if REG_MODE
        case 'D' :      do_dump();      break ;
        case 'F' :      do_file();      break ;
#endif
#if DEMO
#else
        case 'B' :      do_bl_line();   break ;
        case 'H' :      do_header();    break ;
        case 'P' :      do_pause();     break ;
        case 'T' :      do_timer();
        case 'Y' :      do_yes_no();
        case '_' :      do_cursor();    break ;
#endif
        case 'C' :      do_char();
        case 'M' :      do_message();   break ;
        case 'S' :      do_sound();     break ;
        case 'W' :      do_wait();      break ;
        case '^' :      break ;                 // cls() already done
        default  :      do_help();
    }
    bu_quit( 0 );
}
